'use strict';

var view = require('@codemirror/view');
var language = require('@codemirror/language');
var highlight = require('@lezer/highlight');
var _utils = require('@utils');

/**
 * Enhanced Volcano theme color definitions
 * ---------------------------------------------
 * Colors organized by function with visual color blocks
 */
// Base colors
const base00 = '#390000'; // Background
const base01 = '#F8F8F8'; // Foreground
const base02 = '#750000'; // Selection background
const base03 = '#e7c0c0'; // Comments, invisibles
const base04 = '#970000'; // Cursor
const base05 = '#f12727'; // Default foreground
const base06 = '#580000'; // Dark background for panels
const base07 = '#4a0000'; // Darker background (gutter)
// Accent colors
const base_red = '#ec0d1e'; // Errors, invalid
const base_orange = '#aa5507'; // Numbers, constants
const base_yellow = '#fec758'; // Classes, attributes
const base_green = '#9df39f'; // Success
const base_cyan = '#7df3f7'; // Functions, parameters
const base_blue = '#7dcaf7'; // Variables
const base_purple = '#c27df7'; // Keywords, operators
const base_magenta = '#f77dca'; // Special characters
// UI specific colors
const invalid = '#ffffff';
const darkBackground = base06;
const highlightBackground = '#ff000035'; // Line highlight with transparency
const background = base00;
const tooltipBackground = '#680000';
const selection = '#75000080'; // Selection background with transparency
const selectionMatch = '#7500009a'; // Selection match with transparency
const cursor = base04; // Cursor color
const activeBracketBg = '#ff550040'; // Active bracket background with transparency
const activeBracketBorder = base_red; // Active bracket border
const diagnosticWarning = base_orange; // Warning color
const linkColor = base_cyan; // Link color
const visitedLinkColor = base_purple; // Visited link color
/**
 * Enhanced editor theme styles for Volcano
 */
const volcanoTheme = view.EditorView.theme({
    // Base editor styles
    '&': {
        color: base01,
        backgroundColor: background,
        fontSize: _utils.generalContent.fontSize,
        fontFamily: _utils.generalContent.fontFamily,
    },
    // Content and cursor
    '.cm-content': {
        caretColor: cursor,
        lineHeight: _utils.generalContent.lineHeight,
    },
    '.cm-cursor, .cm-dropCursor': {
        borderLeftColor: cursor,
        borderLeftWidth: _utils.generalCursor.borderLeftWidth,
    },
    '.cm-fat-cursor': {
        backgroundColor: `${cursor}99`,
        color: background,
    },
    // Selection
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': {
        backgroundColor: selection,
    },
    // Make sure selection appears above active line
    '.cm-selectionLayer': {
        zIndex: 100,
    },
    // Search functionality
    '.cm-searchMatch': {
        backgroundColor: '#75000080',
        outline: `1px solid ${base_red}`,
        color: base01,
        borderRadius: _utils.generalSearchField.borderRadius,
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base_red,
        color: background,
        padding: _utils.generalSearchField.padding,
        '& span': {
            color: base01,
        },
    },
    '.cm-search.cm-panel.cm-textfield': {
        color: base01,
        borderRadius: _utils.generalSearchField.borderRadius,
        padding: _utils.generalSearchField.padding,
    },
    // Panels
    '.cm-panels': {
        backgroundColor: darkBackground,
        color: base01,
        borderRadius: '4px',
    },
    '.cm-panels.cm-panels-top': {
        borderBottom: `2px solid ${base00}`,
    },
    '.cm-panels.cm-panels-bottom': {
        borderTop: `2px solid ${base00}`,
    },
    '.cm-panel button': {
        backgroundColor: base07,
        color: base01,
        border: _utils.generalPanel.border,
        borderRadius: _utils.generalPanel.borderRadius,
        padding: _utils.generalPanel.padding,
    },
    '.cm-panel button:hover': {
        backgroundColor: base02,
        boxShadow: '0 0 4px #ff550080',
    },
    // Line highlighting
    '.cm-activeLine': {
        backgroundColor: highlightBackground,
        borderRadius: _utils.generalLine.borderRadius,
        zIndex: 1,
    },
    // Gutters
    '.cm-gutters': {
        backgroundColor: base07,
        color: base03,
        border: _utils.generalGutter.border,
        borderRight: `1px solid ${base06}`,
        paddingRight: _utils.generalGutter.paddingRight,
    },
    '.cm-activeLineGutter': {
        backgroundColor: '#550000',
        color: base01,
        fontWeight: _utils.generalGutter.fontWeight,
    },
    '.cm-lineNumbers': {
        fontSize: _utils.generalGutter.fontSize,
    },
    '.cm-foldGutter': {
        fontSize: _utils.generalGutter.fontSize,
    },
    '.cm-foldGutter .cm-gutterElement': {
        color: base03,
        cursor: 'pointer',
    },
    '.cm-foldGutter .cm-gutterElement:hover': {
        color: base01,
    },
    // Tooltips and autocomplete
    '.cm-tooltip': {
        backgroundColor: tooltipBackground,
        border: `1px solid ${base_red}60`,
        borderRadius: _utils.generalTooltip.borderRadius,
        padding: _utils.generalTooltip.padding,
        boxShadow: '0 2px 8px rgba(0, 0, 0, 0.4)',
    },
    '.cm-tooltip-autocomplete': {
        '& > ul': {
            backgroundColor: tooltipBackground,
            border: 'none',
            maxHeight: '300px',
        },
        '& > ul > li': {
            padding: _utils.generalTooltip.padding,
            lineHeight: _utils.generalTooltip.lineHeight,
        },
        '& > ul > li[aria-selected]': {
            backgroundColor: base02,
            color: base01,
            borderRadius: _utils.generalTooltip.borderRadiusSelected,
        },
        '& > ul > li > span.cm-completionIcon': {
            color: base03,
            paddingRight: _utils.generalTooltip.paddingRight,
        },
        '& > ul > li > span.cm-completionDetail': {
            color: base03,
            fontStyle: 'italic',
        },
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent',
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: tooltipBackground,
        borderBottomColor: tooltipBackground,
    },
    // Diagnostics styling
    '.cm-diagnostic': {
        '&-error': {
            borderLeft: `3px solid ${base_red}`,
        },
        '&-warning': {
            borderLeft: `3px solid ${diagnosticWarning}`,
        },
        '&-info': {
            borderLeft: `3px solid ${linkColor}`,
        },
    },
    '.cm-lintPoint-error': {
        borderBottom: `2px wavy ${base_red}`,
    },
    '.cm-lintPoint-warning': {
        borderBottom: `2px wavy ${diagnosticWarning}`,
    },
    // Matching brackets
    '.cm-matchingBracket': {
        backgroundColor: activeBracketBg,
        outline: `1px solid ${activeBracketBorder}`,
        borderRadius: _utils.generalMatching.borderRadius,
    },
    '.cm-nonmatchingBracket': {
        backgroundColor: `${base_red}40`,
        outline: `1px solid ${invalid}`,
        borderRadius: _utils.generalMatching.borderRadius,
    },
    // Selection matches
    '.cm-selectionMatch': {
        backgroundColor: selectionMatch,
        outline: `1px solid ${base02}50`,
        borderRadius: _utils.generalMatching.borderRadius,
    },
    // Fold placeholder
    '.cm-foldPlaceholder': {
        backgroundColor: tooltipBackground,
        color: base03,
        border: `1px dotted ${base03}70`,
        borderRadius: _utils.generalPlaceholder.borderRadius,
        padding: _utils.generalPlaceholder.padding,
        margin: _utils.generalPlaceholder.margin,
    },
    // Focus outline
    '&.cm-focused': {
        outline: 'none',
        boxShadow: `0 0 0 2px ${background}, 0 0 0 3px ${base_red}70`,
    },
    // Scrollbars
    '& .cm-scroller::-webkit-scrollbar': {
        width: _utils.generalScroller.width,
        height: _utils.generalScroller.height,
    },
    '& .cm-scroller::-webkit-scrollbar-track': {
        background: base07,
    },
    '& .cm-scroller::-webkit-scrollbar-thumb': {
        backgroundColor: base02,
        borderRadius: _utils.generalScroller.borderRadius,
        border: `3px solid ${base07}`,
    },
    '& .cm-scroller::-webkit-scrollbar-thumb:hover': {
        backgroundColor: base_red,
    },
    // Ghost text
    '.cm-ghostText': {
        opacity: '0.5',
        color: base03,
    },
}, { dark: true });
/**
 * Enhanced syntax highlighting for Volcano theme
 */
const volcanoHighlightStyle = language.HighlightStyle.define([
    // Keywords and control flow
    { tag: highlight.tags.keyword, color: base05, fontWeight: 'bold' },
    { tag: highlight.tags.controlKeyword, color: base05, fontWeight: 'bold' },
    { tag: highlight.tags.moduleKeyword, color: base05, fontWeight: 'bold' },
    // Names and variables
    { tag: [highlight.tags.name, highlight.tags.deleted, highlight.tags.character, highlight.tags.macroName], color: base01 },
    { tag: [highlight.tags.variableName], color: base_yellow },
    { tag: [highlight.tags.propertyName], color: base_cyan, fontStyle: 'normal' },
    // Classes and types
    { tag: [highlight.tags.typeName], color: base_green },
    { tag: [highlight.tags.className], color: base_yellow, fontStyle: 'italic' },
    { tag: [highlight.tags.namespace], color: base_blue, fontStyle: 'italic' },
    // Operators and punctuation
    { tag: [highlight.tags.operator, highlight.tags.operatorKeyword], color: base_purple },
    { tag: [highlight.tags.bracket], color: base03 },
    { tag: [highlight.tags.brace], color: base03 },
    { tag: [highlight.tags.punctuation], color: base03 },
    // Functions and parameters
    { tag: [highlight.tags.function(highlight.tags.variableName)], color: base_cyan },
    { tag: [highlight.tags.labelName], color: base_cyan, fontStyle: 'italic' },
    { tag: [highlight.tags.definition(highlight.tags.function(highlight.tags.variableName))], color: base_cyan },
    { tag: [highlight.tags.definition(highlight.tags.variableName)], color: base_yellow },
    // Constants and literals
    { tag: highlight.tags.number, color: base_orange },
    { tag: highlight.tags.changed, color: base_orange },
    { tag: highlight.tags.annotation, color: invalid, fontStyle: 'italic' },
    { tag: highlight.tags.modifier, color: base_orange, fontStyle: 'italic' },
    { tag: highlight.tags.self, color: base_orange },
    {
        tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)],
        color: base_orange,
    },
    { tag: [highlight.tags.atom, highlight.tags.bool, highlight.tags.special(highlight.tags.variableName)], color: base_orange },
    // Strings and regex
    { tag: [highlight.tags.processingInstruction, highlight.tags.inserted], color: base_green },
    { tag: [highlight.tags.special(highlight.tags.string), highlight.tags.regexp], color: base_magenta },
    { tag: highlight.tags.string, color: base_yellow },
    // Punctuation and structure
    { tag: highlight.tags.definition(highlight.tags.typeName), color: base_green, fontWeight: 'bold' },
    { tag: [highlight.tags.definition(highlight.tags.name), highlight.tags.separator], color: base01 },
    // Comments and documentation
    { tag: highlight.tags.meta, color: base03 },
    { tag: highlight.tags.comment, fontStyle: 'italic', color: base03 },
    { tag: highlight.tags.docComment, fontStyle: 'italic', color: base03 },
    // HTML/XML elements
    { tag: [highlight.tags.tagName], color: base_red },
    { tag: [highlight.tags.attributeName], color: base_yellow },
    // Markdown and text formatting
    { tag: [highlight.tags.heading], fontWeight: 'bold', color: base_orange },
    { tag: highlight.tags.heading1, color: base_red, fontWeight: 'bold' },
    { tag: highlight.tags.heading2, color: base_orange },
    { tag: highlight.tags.heading3, color: base_yellow },
    { tag: highlight.tags.heading4, color: base_green },
    { tag: highlight.tags.heading5, color: base_cyan },
    { tag: highlight.tags.heading6, color: base_blue },
    { tag: [highlight.tags.strong], fontWeight: 'bold', color: base01 },
    { tag: [highlight.tags.emphasis], fontStyle: 'italic', color: base_yellow },
    // Links and URLs
    {
        tag: [highlight.tags.link],
        color: visitedLinkColor,
        fontWeight: '500',
        textDecoration: 'underline',
        textUnderlinePosition: 'under',
    },
    {
        tag: [highlight.tags.url],
        color: linkColor,
        textDecoration: 'underline',
        textUnderlineOffset: '2px',
    },
    // Special states
    {
        tag: [highlight.tags.invalid],
        color: base01,
        textDecoration: 'underline wavy',
        borderBottom: `1px wavy ${invalid}`,
    },
    { tag: [highlight.tags.strikethrough], color: invalid, textDecoration: 'line-through' },
    // Enhanced syntax highlighting
    { tag: highlight.tags.constant(highlight.tags.name), color: base_orange },
    { tag: highlight.tags.deleted, color: base_red },
    { tag: highlight.tags.squareBracket, color: base03 },
    { tag: highlight.tags.angleBracket, color: base03 },
    // Additional specific styles
    { tag: highlight.tags.monospace, color: base01 },
    { tag: [highlight.tags.contentSeparator], color: base_blue },
    { tag: highlight.tags.quote, color: base03 },
]);
/**
 * Combined Volcano theme extension
 */
const volcano = [
    volcanoTheme,
    language.syntaxHighlighting(volcanoHighlightStyle),
];

exports.volcano = volcano;
exports.volcanoHighlightStyle = volcanoHighlightStyle;
exports.volcanoTheme = volcanoTheme;
